"{ Package: 'stx:goodies/ston' }"

"{ NameSpace: Smalltalk }"

LibraryDefinition subclass:#stx_goodies_ston
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'* Projects & Packages *'
!


!stx_goodies_ston class methodsFor:'description'!

excludedFromPreRequisites
    "obsolete; temporarily, this is still called for, but will eventually vanish.
    
     List packages which are to be explicitely excluded from the automatic constructed
     prerequisites lists (both). 
     If empty, everything that is found along the inheritance of any of
     my classes is considered to be a prerequisite package."

    ^ #(
    )
!

mandatoryPreRequisites
    "list packages which are mandatory as a prerequisite.
     This are packages containing superclasses of my classes and classes which
     are extended by myself.
     They are mandatory, because we need these packages as a prerequisite for loading and compiling.
     This method is generated automatically,
     by searching along the inheritance chain of all of my classes.
     Please take a look at the #referencedPreRequisites method as well."

    ^ #(
        #'stx:libbasic'    "AbstractTime - extended"
        #'stx:libbasic2'    "RunArray - extended"
        #'stx:libcompat'    "FileReference - superclass of STONFileReference"
        #'stx:libhtml'    "URL - extended"
        #'stx:libview'    "Color - extended"
    )
!

referencedPreRequisites
    "list packages which are a prerequisite, because they contain
     classes which are referenced by my classes.
     These packages are NOT needed as a prerequisite for compiling or loading,
     however, a class from it may be referenced during execution and having it
     unloaded then may lead to a runtime doesNotUnderstand error, unless the caller
     includes explicit checks for the package being present.
     This method is generated automatically,
     by searching all classes (and their packages) which are referenced by my classes.
     Please also take a look at the #mandatoryPreRequisites method"

    ^ #(
    )
!

subProjects
    "list packages which are known as subprojects.
     The generated makefile will enter those and make there as well.
     However: they are not forced to be loaded when a package is loaded;
     for those, redefine #referencedPrerequisites or #mandatoryPreRequisites."

    ^ #(
    )
! !

!stx_goodies_ston class methodsFor:'description - contents'!

classNamesAndAttributes
    "lists the classes which are to be included in the project.
     Each entry in the list may be: a single class-name (symbol),
     or an array-literal consisting of class name and attributes.
     Attributes are: #autoload or #<os> where os is one of win32, unix,..."

    ^ #(
        "<className> or (<className> attributes...) in load order"
        STON
        STONCStyleCommentsSkipStream
        STONFileReference
        STONJSON
        STONReader
        STONReaderError
        STONReference
        STONStreamWriter
        STONWriter
        STONWriterError
        #'stx_goodies_ston'
        STONListWriter
        STONMapWriter
        STONShortListWriter
    )
!

extensionMethodNames
    "lists the extension methods which are to be included in the project.
     Entries are 2-element array literals, consisting of class-name and selector.
     A correponding method with real names must be present in my concrete subclasses
     if it has extensions."

    ^ #(
        Association stonOn:
        Bag stonOn:
        Boolean stonContainSubObjects
        Boolean stonOn:
        ByteArray stonContainSubObjects
        ByteArray stonOn:
        Character stonOn:
        Class stonOn:
        ClassDescription stonContainSubObjects
        Collection stonOn:
        Color stonContainSubObjects
        Color stonOn:
        Date stonContainSubObjects
        Date stonOn:
        Dictionary stonOn:
        Dictionary stonPostReferenceResolution
        Fraction stonOn:
        Integer stonOn:
        Interval fromSton:
        Interval stonOn:
        Metaclass stonName
        Metaclass stonOn:
        Number stonContainSubObjects
        Number stonOn:
        Object fromSton:
        Object isStonReference
        Object stonContainSubObjects
        Object stonOn:
        Object stonPostReferenceResolution
        Object stonProcessSubObjects:
        Object stonShouldWriteNilInstVars
        OrderedDictionary stonOn:
        Point fromSton:
        Point stonOn:
        SequenceableCollection stonOn:
        Set stonPostReferenceResolution
        String stonContainSubObjects
        String stonOn:
        Symbol stonOn:
        Time stonContainSubObjects
        Time stonOn:
        Timestamp stonContainSubObjects
        Timestamp stonOn:
        UndefinedObject stonContainSubObjects
        UndefinedObject stonOn:
        'Bag class' fromSton:
        'ByteArray class' fromSton:
        'Character class' fromSton:
        'Class class' fromSton:
        'Collection class' fromSton:
        'Color class' fromSton:
        'Date class' fromSton:
        'Dictionary class' fromSton:
        'Interval class' fromSton:
        'Metaclass class' fromSton:
        'Object class' fromSton:
        'Object class' stonAllInstVarNames
        'Object class' stonName
        'OrderedDictionary class' fromSton:
        'RunArray class' fromSton:
        'SequenceableCollection class' fromSton:
        'Time class' fromSton:
        'Timestamp class' fromSton:
        Filename stonContainSubObjects
        Filename stonOn:
        'URL class' fromSton:
        'CharacterArray class' findFirstInString:inSet:startingAt:
    )
! !

!stx_goodies_ston class methodsFor:'description - project information'!

companyName
    "Returns a company string which will appear in <lib>.rc.
     Under win32, this is placed into the dll's file-info.
     Other systems may put it elsewhere, or ignore it."

    ^ 'Claus Gittinger & eXept Software AG'
!

description
    "Returns a description string which will appear in nt.def / bc.def"

    ^ 'Smalltalk/X Class library'
!

legalCopyright
    "Returns a copyright string which will appear in <lib>.rc.
     Under win32, this is placed into the dll's file-info.
     Other systems may put it elsewhere, or ignore it."

    ^ 'Copyright Claus Gittinger 2019\nCopyright eXept Software AG 2019'
!

productName
    "Returns a product name which will appear in <lib>.rc.
     Under win32, this is placed into the dll's file-info.
     This method is usually redefined in a concrete application definition"

    ^ 'Smalltalk/X'
! !

!stx_goodies_ston class methodsFor:'documentation'!

version_HG
    ^ '$Changeset: <not expanded> $'
! !

